@@ -28,6 +28,7 @@
 
 #include <ell/ell.h>
 #include "src/dbus.h"
+#include "src/netconfig.h"
 #include "src/agent.h"
 #include "src/iwd.h"
 #include "src/module.h"
@@ -69,29 +70,12 @@ static struct l_queue *agents;
  * This should probably be configurable by user via
  * config file/command line option/env variable.
  */
+#ifdef HAVE_DBUS
 static unsigned int agent_timeout_input_request(void)
 {
 	return 120;
 }
 
-static void send_request(struct agent *agent, const char *request)
-{
-	struct l_dbus_message *message;
-
-	l_debug("send %s request to %s %s", request, agent->owner,
-							agent->path);
-
-	message = l_dbus_message_new_method_call(dbus_get_bus(),
-							agent->owner,
-							agent->path,
-							IWD_AGENT_INTERFACE,
-							request);
-
-	l_dbus_message_set_arguments(message, "");
-
-	l_dbus_send(dbus_get_bus(), message);
-}
-
 static void send_cancel_request(void *user_data, int reason)
 {
 	struct agent *agent = user_data;
@@ -125,14 +109,17 @@ static void send_cancel_request(void *user_data, int reason)
 							"Cancel");
 
 	l_dbus_message_set_arguments(message, "s", reasonstr);
+	l_dbus_message_set_no_reply(message, true);
 
 	l_dbus_send(dbus_get_bus(), message);
 }
+#endif
 
 static void agent_request_free(void *user_data)
 {
 	struct agent_request *request = user_data;
 
+#ifdef HAVE_DBUS
 	l_dbus_message_unref(request->message);
 
 	if (request->trigger)
@@ -141,10 +128,12 @@ static void agent_request_free(void *user_data)
 
 	if (request->destroy)
 		request->destroy(request->user_data);
+#endif
 
 	l_free(request);
 }
 
+#ifdef HAVE_DBUS
 static void passphrase_reply(struct l_dbus_message *reply,
 					struct agent_request *request)
 {
@@ -187,6 +176,7 @@ done:
 	user_callback(result, username, passwd,
 			request->trigger, request->user_data);
 }
+#endif
 
 static void agent_finalize_pending(struct agent *agent,
 						struct l_dbus_message *reply)
@@ -198,6 +188,7 @@ static void agent_finalize_pending(struct agent *agent,
 		agent->timeout = NULL;
 	}
 
+#ifdef HAVE_DBUS
 	pending = l_queue_pop_head(agent->requests);
 
 	switch (pending->type) {
@@ -215,6 +206,7 @@ static void agent_finalize_pending(struct agent *agent,
 	}
 
 	agent_request_free(pending);
+#endif
 }
 
 static void agent_free(void *data)
@@ -226,13 +218,17 @@ static void agent_free(void *data)
 	if (agent->timeout)
 		l_timeout_remove(agent->timeout);
 
+#ifdef HAVE_DBUS
 	if (agent->pending_id)
 		l_dbus_cancel(dbus_get_bus(), agent->pending_id);
 
 	l_queue_destroy(agent->requests, agent_request_free);
+#endif
 
+#ifdef HAVE_DBUS
 	if (agent->disconnect_watch)
 		l_dbus_remove_watch(dbus_get_bus(), agent->disconnect_watch);
+#endif
 
 	l_free(agent->owner);
 	l_free(agent->path);
@@ -245,15 +241,18 @@ static void request_timeout(struct l_timeout *timeout, void *user_data)
 {
 	struct agent *agent = user_data;
 
+#ifdef HAVE_DBUS
 	l_dbus_cancel(dbus_get_bus(), agent->pending_id);
 
 	send_cancel_request(agent, -ETIMEDOUT);
+#endif
 
 	agent_finalize_pending(agent, NULL);
 
 	agent_send_next_request(agent);
 }
 
+#ifdef HAVE_DBUS
 static void agent_receive_reply(struct l_dbus_message *message,
 							void *user_data)
 {
@@ -268,6 +267,7 @@ static void agent_receive_reply(struct l_dbus_message *message,
 	if (!agent->pending_id)
 		agent_send_next_request(agent);
 }
+#endif
 
 static void agent_send_next_request(struct agent *agent)
 {
@@ -283,16 +283,17 @@ static void agent_send_next_request(struct agent *agent)
 
 	l_debug("send request to %s %s", agent->owner, agent->path);
 
+#ifdef HAVE_DBUS
 	agent->pending_id = l_dbus_send_with_reply(dbus_get_bus(),
 							pending->message,
 							agent_receive_reply,
 							agent, NULL);
+#endif
 
 	pending->message = NULL;
-
-	return;
 }
 
+#ifdef HAVE_DBUS
 static unsigned int agent_queue_request(struct agent *agent,
 					enum agent_request_type type,
 					struct l_dbus_message *message,
@@ -351,6 +352,7 @@ static struct agent *get_agent(const char *owner)
 
 	return l_queue_peek_head(agents);
 }
+#endif
 
 /**
  * agent_request_passphrase:
@@ -375,6 +377,7 @@ unsigned int agent_request_passphrase(const char *path,
 				void *user_data,
 				agent_request_destroy_func_t destroy)
 {
+#ifdef HAVE_DBUS
 	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
 	struct l_dbus_message *message;
 
@@ -394,6 +397,9 @@ unsigned int agent_request_passphrase(const char *path,
 	return agent_queue_request(agent, AGENT_REQUEST_TYPE_PASSPHRASE,
 					message, agent_timeout_input_request(),
 					callback, trigger, user_data, destroy);
+#else
+    return 0;
+#endif
 }
 
 unsigned int agent_request_pkey_passphrase(const char *path,
@@ -402,6 +408,7 @@ unsigned int agent_request_pkey_passphrase(const char *path,
 				void *user_data,
 				agent_request_destroy_func_t destroy)
 {
+#ifdef HAVE_DBUS
 	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
 	struct l_dbus_message *message;
 
@@ -420,6 +427,9 @@ unsigned int agent_request_pkey_passphrase(const char *path,
 	return agent_queue_request(agent, AGENT_REQUEST_TYPE_PASSPHRASE,
 					message, agent_timeout_input_request(),
 					callback, trigger, user_data, destroy);
+#else
+    return 0;
+#endif
 }
 
 unsigned int agent_request_user_name_password(const char *path,
@@ -428,6 +438,7 @@ unsigned int agent_request_user_name_password(const char *path,
 				void *user_data,
 				agent_request_destroy_func_t destroy)
 {
+#ifdef HAVE_DBUS
 	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
 	struct l_dbus_message *message;
 
@@ -446,6 +457,9 @@ unsigned int agent_request_user_name_password(const char *path,
 	return agent_queue_request(agent, AGENT_REQUEST_TYPE_USER_NAME_PASSWD,
 					message, agent_timeout_input_request(),
 					callback, trigger, user_data, destroy);
+#else
+    return 0;
+#endif
 }
 
 unsigned int agent_request_user_password(const char *path, const char *user,
@@ -453,6 +467,7 @@ unsigned int agent_request_user_password(const char *path, const char *user,
 				struct l_dbus_message *trigger, void *user_data,
 				agent_request_destroy_func_t destroy)
 {
+#ifdef HAVE_DBUS
 	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
 	struct l_dbus_message *message;
 
@@ -471,6 +486,9 @@ unsigned int agent_request_user_password(const char *path, const char *user,
 	return agent_queue_request(agent, AGENT_REQUEST_TYPE_PASSPHRASE,
 					message, agent_timeout_input_request(),
 					callback, trigger, user_data, destroy);
+#else
+    return 0;
+#endif
 }
 
 static bool find_request(const void *a, const void *b)
@@ -500,9 +518,11 @@ bool agent_request_cancel(unsigned int req_id, int reason)
 		return false;
 
 	if (!request->message) {
+#ifdef HAVE_DBUS
 		send_cancel_request(agent, reason);
 
 		l_dbus_cancel(dbus_get_bus(), agent->pending_id);
+#endif
 
 		agent->pending_id = 0;
 
@@ -519,6 +539,7 @@ bool agent_request_cancel(unsigned int req_id, int reason)
 	return true;
 }
 
+#ifdef HAVE_DBUS
 static void agent_disconnect(struct l_dbus *dbus, void *user_data)
 {
 	struct agent *agent = user_data;
@@ -603,6 +624,60 @@ static struct l_dbus_message *agent_unregister(struct l_dbus *dbus,
 	return reply;
 }
 
+static struct l_dbus_message *netconfig_agent_register(struct l_dbus *dbus,
+						struct l_dbus_message *message,
+						void *user_data)
+{
+	struct l_dbus_message *reply;
+	const char *path;
+	int r;
+
+	l_debug("");
+
+	if (!l_dbus_message_get_arguments(message, "o", &path))
+		return dbus_error_invalid_args(message);
+
+	if (!netconfig_enabled())
+		return dbus_error_not_supported(message);
+
+	r = netconfig_register_agent(l_dbus_message_get_sender(message), path);
+	if (r)
+		return dbus_error_from_errno(r, message);
+
+	l_debug("agent %s path %s",
+		l_dbus_message_get_sender(message), path);
+
+	reply = l_dbus_message_new_method_return(message);
+	l_dbus_message_set_arguments(reply, "");
+	return reply;
+}
+
+static struct l_dbus_message *netconfig_agent_unregister(struct l_dbus *dbus,
+						struct l_dbus_message *message,
+					void *user_data)
+{
+	struct l_dbus_message *reply;
+	const char *path;
+	int r;
+
+	l_debug("");
+
+	if (!l_dbus_message_get_arguments(message, "o", &path))
+		return dbus_error_invalid_args(message);
+
+	if (!netconfig_enabled())
+		return dbus_error_not_supported(message);
+
+	r = netconfig_unregister_agent(l_dbus_message_get_sender(message),
+					path);
+	if (r)
+		return dbus_error_from_errno(r, message);
+
+	reply = l_dbus_message_new_method_return(message);
+	l_dbus_message_set_arguments(reply, "");
+	return reply;
+}
+
 static void setup_agent_interface(struct l_dbus_interface *interface)
 {
 	l_dbus_interface_method(interface, "RegisterAgent", 0,
@@ -611,13 +686,34 @@ static void setup_agent_interface(struct l_dbus_interface *interface)
 	l_dbus_interface_method(interface, "UnregisterAgent", 0,
 				agent_unregister,
 				"", "o", "path");
+
+	l_dbus_interface_method(interface,
+				"RegisterNetworkConfigurationAgent", 0,
+				netconfig_agent_register, "", "o", "path");
+	l_dbus_interface_method(interface,
+				"UnregisterNetworkConfigurationAgent", 0,
+				netconfig_agent_unregister, "", "o", "path");
 }
+#endif
 
 static bool release_agent(void *data, void *user_data)
 {
 	struct agent *agent = data;
+#ifdef HAVE_DBUS
+	struct l_dbus_message *message;
 
-	send_request(agent, "Release");
+	l_debug("send Release to %s %s", agent->owner, agent->path);
+
+	message = l_dbus_message_new_method_call(dbus_get_bus(),
+							agent->owner,
+							agent->path,
+							IWD_AGENT_INTERFACE,
+							"Release");
+
+	l_dbus_message_set_arguments(message, "");
+	l_dbus_message_set_no_reply(message, true);
+	l_dbus_send(dbus_get_bus(), message);
+#endif
 
 	agent_free(agent);
 
@@ -626,10 +722,13 @@ static bool release_agent(void *data, void *user_data)
 
 static int agent_init(void)
 {
+#ifdef HAVE_DBUS
 	struct l_dbus *dbus = dbus_get_bus();
+#endif
 
 	agents = l_queue_new();
 
+#ifdef HAVE_DBUS
 	if (!l_dbus_register_interface(dbus, IWD_AGENT_MANAGER_INTERFACE,
 						setup_agent_interface,
 						NULL, false)) {
@@ -646,15 +745,18 @@ static int agent_init(void)
 		l_dbus_unregister_interface(dbus, IWD_AGENT_MANAGER_INTERFACE);
 		return -EIO;
 	}
+#endif
 
 	return 0;
 }
 
 static void agent_exit(void)
 {
+#ifdef HAVE_DBUS
 	struct l_dbus *dbus = dbus_get_bus();
 
 	l_dbus_unregister_interface(dbus, IWD_AGENT_MANAGER_INTERFACE);
+#endif
 
 	l_queue_destroy(agents, agent_free);
 	agents = NULL;
