@@ -29,8 +29,14 @@
 #include <errno.h>
 #include <getopt.h>
 #include <signal.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
 #include <ell/ell.h>
 
+#include <ell/useful.h>
+
 #include "linux/nl80211.h"
 
 #include "src/iwd.h"
@@ -42,6 +48,8 @@
 #include "src/rfkill.h"
 #include "src/storage.h"
 #include "src/anqp.h"
+#include "src/netconfig.h"
+#include "src/crypto.h"
 
 #include "src/backtrace.h"
 
@@ -54,6 +62,7 @@ static const char *nointerfaces;
 static const char *phys;
 static const char *nophys;
 static const char *debugopt;
+static bool developeropt;
 static bool terminating;
 static bool nl80211_complete;
 
@@ -74,7 +83,9 @@ static void iwd_shutdown(void)
 		return;
 	}
 
+#ifdef HAVE_DBUS
 	dbus_shutdown();
+#endif
 	netdev_shutdown();
 
 	timeout = l_timeout_create(1, main_loop_quit, NULL, NULL);
@@ -126,13 +137,18 @@ const char *iwd_get_phy_blacklist(void)
 	return nophys;
 }
 
+bool iwd_is_developer_mode(void)
+{
+	return developeropt;
+}
+
 static void usage(void)
 {
 	printf("iwd - Wireless daemon\n"
 		"Usage:\n");
 	printf("\tiwd [options]\n");
 	printf("Options:\n"
-		"\t-B, --dbus-debug       Enable D-Bus debugging\n"
+		"\t-E, --developer        Enable developer mode\n"
 		"\t-i, --interfaces       Interfaces to manage\n"
 		"\t-I, --nointerfaces     Interfaces to ignore\n"
 		"\t-p, --phys             Phys to manage\n"
@@ -143,7 +159,7 @@ static void usage(void)
 }
 
 static const struct option main_options[] = {
-	{ "dbus-debug",   no_argument,       NULL, 'B' },
+	{ "developer",    no_argument,       NULL, 'E' },
 	{ "version",      no_argument,       NULL, 'v' },
 	{ "interfaces",   required_argument, NULL, 'i' },
 	{ "nointerfaces", required_argument, NULL, 'I' },
@@ -174,6 +190,7 @@ static void nl80211_appeared(const struct l_genl_family_info *info,
 	}
 }
 
+#ifdef HAVE_DBUS
 static void request_name_callback(struct l_dbus *dbus, bool success,
 					bool queued, void *user_data)
 {
@@ -185,6 +202,16 @@ static void request_name_callback(struct l_dbus *dbus, bool success,
 	if (!l_dbus_object_manager_enable(dbus, "/"))
 		l_warn("Unable to register the ObjectManager");
 
+	if (!l_dbus_object_add_interface(dbus, IWD_BASE_PATH,
+						IWD_DAEMON_INTERFACE,
+						NULL) ||
+			!l_dbus_object_add_interface(dbus, IWD_BASE_PATH,
+						L_DBUS_INTERFACE_PROPERTIES,
+						NULL))
+		l_info("Unable to add %s and/or %s at %s",
+			IWD_DAEMON_INTERFACE, L_DBUS_INTERFACE_PROPERTIES,
+			IWD_BASE_PATH);
+
 	/* TODO: Always request nl80211 for now, ignoring auto-loading */
 	l_genl_request_family(genl, NL80211_GENL_NAME, nl80211_appeared,
 				NULL, NULL);
@@ -194,12 +221,39 @@ fail_exit:
 	l_main_quit();
 }
 
+static struct l_dbus_message *iwd_dbus_get_info(struct l_dbus *dbus,
+						struct l_dbus_message *message,
+						void *user_data)
+{
+	struct l_dbus_message *reply =
+				l_dbus_message_new_method_return(message);
+	_auto_(l_free) char *storage_dir = storage_get_path(NULL);
+
+	l_dbus_message_set_arguments(reply, "a{sv}", 3,
+					"NetworkConfigurationEnabled",
+					"b", netconfig_enabled(),
+					"StateDirectory", "s", storage_dir,
+					"Version", "s", VERSION);
+
+	return reply;
+}
+
+static void iwd_setup_deamon_interface(struct l_dbus_interface *interface)
+{
+	l_dbus_interface_method(interface, "GetInfo", 0, iwd_dbus_get_info,
+				"a{sv}", "", "info");
+}
+
 static void dbus_ready(void *user_data)
 {
 	struct l_dbus *dbus = user_data;
 
 	l_dbus_name_acquire(dbus, "net.connman.iwd", false, false, false,
 				request_name_callback, NULL);
+
+	l_dbus_register_interface(dbus, IWD_DAEMON_INTERFACE,
+					iwd_setup_deamon_interface,
+					NULL, false);
 }
 
 static void dbus_disconnected(void *user_data)
@@ -207,6 +261,7 @@ static void dbus_disconnected(void *user_data)
 	l_info("D-Bus disconnected, quitting...");
 	iwd_shutdown();
 }
+#endif
 
 static void print_koption(const void *key, void *value, void *user_data)
 {
@@ -219,7 +274,7 @@ static void print_koption(const void *key, void *value, void *user_data)
 #define ADD_OPTIONAL(opt) \
 	l_hashmap_replace(optional, opt, &r, NULL)
 
-static int check_crypto()
+static int check_crypto(void)
 {
 	int r = 0;
 	struct l_hashmap *options = l_hashmap_string_new();
@@ -351,9 +406,68 @@ done:
 	return r;
 }
 
+/*
+ * Initialize a systemd encryption key for encrypting/decrypting credentials.
+ */
+static bool setup_system_key(void)
+{
+	int fd;
+	struct stat st;
+	const char *cred_dir;
+	void *key = NULL;
+	_auto_(l_free) char *path = NULL;
+	_auto_(l_free) char *key_id = NULL;
+	bool r = false;
+
+	key_id = l_settings_get_string(iwd_config, "General",
+							"SystemdEncrypt");
+	if (!key_id)
+		return true;
+
+	cred_dir = getenv("CREDENTIALS_DIRECTORY");
+	if (!cred_dir) {
+		l_warn("SystemdEncrypt enabled but CREDENTIALS_DIRECTORY not "
+			"set, check iwd.service file");
+		return false;
+	}
+
+	path = l_strdup_printf("%s/%s", cred_dir, key_id);
+
+	fd = open(path, O_RDONLY, 0);
+	if (fd < 0) {
+		l_warn("SystemdEncrypt: Cannot open secret: %s (%d)",
+				strerror(errno), errno);
+		return false;
+	}
+
+	if (fstat(fd, &st) < 0 || st.st_size == 0)
+		goto close_fd;
+
+	key = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
+	if (key == MAP_FAILED) {
+		l_warn("SystemdEncrypt: can't mmap secret: %s (%d)",
+				strerror(errno), errno);
+		goto close_fd;
+	}
+
+	if (mlock(key, st.st_size) < 0) {
+		l_warn("SystemdEncrypt: Failed to mlock secrets file");
+		goto unmap;
+	}
+
+	r = storage_init(key, st.st_size);
+	munlock(key, st.st_size);
+
+unmap:
+	munmap(key, st.st_size);
+close_fd:
+	close(fd);
+
+	return r;
+}
+
 int main(int argc, char *argv[])
 {
-	bool enable_dbus_debug = false;
 	int exit_status;
 	struct l_dbus *dbus;
 	const char *config_dir;
@@ -363,14 +477,14 @@ int main(int argc, char *argv[])
 	for (;;) {
 		int opt;
 
-		opt = getopt_long(argc, argv, "Bi:I:p:P:d::vh",
+		opt = getopt_long(argc, argv, "Ei:I:p:P:d::vh",
 							main_options, NULL);
 		if (opt < 0)
 			break;
 
 		switch (opt) {
-		case 'B':
-			enable_dbus_debug = true;
+		case 'E':
+			developeropt = true;
 			break;
 		case 'i':
 			interfaces = optarg;
@@ -474,24 +588,36 @@ int main(int argc, char *argv[])
 	if (getenv("IWD_RTNL_DEBUG"))
 		l_netlink_set_debug(rtnl, do_debug, "[RTNL] ", NULL);
 
+#ifdef HAVE_DBUS
 	dbus = l_dbus_new_default(L_DBUS_SYSTEM_BUS);
 	if (!dbus) {
 		l_error("Failed to initialize D-Bus");
 		goto failed_dbus;
 	}
 
-	if (enable_dbus_debug)
-		l_dbus_set_debug(dbus, do_debug, "[DBUS] ", NULL);
-
 	l_dbus_set_ready_handler(dbus, dbus_ready, dbus, NULL);
 	l_dbus_set_disconnect_handler(dbus, dbus_disconnected, NULL, NULL);
 	dbus_init(dbus);
+#else
+	l_genl_request_family(genl, NL80211_GENL_NAME, nl80211_appeared,
+				NULL, NULL);
+#endif
+
+	if (!setup_system_key())
+		goto failed_storage;
 
 	exit_status = l_main_run_with_signal(signal_handler, NULL);
 
 	iwd_modules_exit();
+	storage_exit();
+
+failed_storage:
+#ifdef HAVE_DBUS
 	dbus_exit();
 	l_dbus_destroy(dbus);
+#else
+(void)NULL;
+#endif
 
 failed_dbus:
 	l_netlink_destroy(rtnl);
